# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: pilot_api.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='pilot_api.proto',
  package='pilot_api',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x0fpilot_api.proto\x12\tpilot_api\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1egoogle/protobuf/wrappers.proto\"2\n\x0c\x44\x65\x62ugModeMsg\x12\"\n\x04mode\x18\x01 \x01(\x0e\x32\x14.pilot_api.DebugMode\"\xc1\x01\n\nMetricInfo\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x15\n\rfriendly_name\x18\x03 \x01(\t\x12\x10\n\x08location\x18\x04 \x01(\t\x12*\n\x0bmetric_type\x18\x05 \x01(\x0e\x32\x15.pilot_api.MetricType\x12\x13\n\x0b\x64\x65scription\x18\x06 \x01(\t\x12\x11\n\tprecision\x18\x07 \x01(\r\x12\x0c\n\x04unit\x18\x08 \x01(\t\x12\r\n\x05group\x18\t \x01(\t\"\"\n\x10\x46wConfigResponse\x12\x0e\n\x06\x63onfig\x18\x01 \x01(\t\"(\n\nIdResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0c\n\x04uuid\x18\x02 \x01(\t\"\x1b\n\x0bUserRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x9c\x01\n\tMetricMsg\x12\x0e\n\x04\x62ool\x18\x01 \x01(\x08H\x00\x12\x10\n\x06\x64ouble\x18\x02 \x01(\x01H\x00\x12\x0f\n\x05\x66loat\x18\x03 \x01(\x02H\x00\x12\x0f\n\x05int32\x18\x04 \x01(\x11H\x00\x12\x0f\n\x05int64\x18\x05 \x01(\x12H\x00\x12\x10\n\x06uint32\x18\x06 \x01(\rH\x00\x12\x10\n\x06uint64\x18\x07 \x01(\x04H\x00\x12\r\n\x03str\x18\x08 \x01(\tH\x00\x42\x07\n\x05value\"q\n\x0f\x46irmwareRequest\x12)\n\x06target\x18\x01 \x01(\x0e\x32\x19.pilot_api.FirmwareTarget\x12\x0e\n\x06\x62inary\x18\x02 \x01(\x0c\x12\x11\n\tvariables\x18\x03 \x01(\t\x12\x10\n\x08\x66wconfig\x18\x04 \x01(\t\"\"\n\x0e\x46irmwareStatus\x12\x10\n\x08progress\x18\x01 \x01(\r*9\n\tDebugMode\x12\x0c\n\x08\x44\x65\x62ugOff\x10\x00\x12\x10\n\x0c\x44\x65\x62ugModules\x10\x01\x12\x0c\n\x08\x44\x65\x62ugAll\x10\x02* \n\nMetricType\x12\x06\n\x02IO\x10\x00\x12\n\n\x06Sensor\x10\x01*4\n\x0e\x46irmwareTarget\x12\x12\n\x0ePilotMainboard\x10\x00\x12\x0e\n\nSmartPower\x10\x01\x32\xed\x04\n\x08PilotApi\x12\x42\n\x0bGetFwConfig\x12\x16.google.protobuf.Empty\x1a\x1b.pilot_api.FwConfigResponse\x12\x36\n\x05GetID\x12\x16.google.protobuf.Empty\x1a\x15.pilot_api.IdResponse\x12\x43\n\x07SetName\x12\x1c.google.protobuf.StringValue\x1a\x1a.google.protobuf.BoolValue\x12\x43\n\x0cSetDebugMode\x12\x17.pilot_api.DebugModeMsg\x1a\x1a.google.protobuf.BoolValue\x12?\n\x0cGetDebugMode\x12\x16.google.protobuf.Empty\x1a\x17.pilot_api.DebugModeMsg\x12@\n\rGetMetricInfo\x12\x16.google.protobuf.Empty\x1a\x15.pilot_api.MetricInfo0\x01\x12M\n\x0eGetFloatMetric\x12\x1c.google.protobuf.StringValue\x1a\x1b.google.protobuf.FloatValue0\x01\x12\x41\n\tGetMetric\x12\x1c.google.protobuf.StringValue\x1a\x14.pilot_api.MetricMsg0\x01\x12\x46\n\x0bSetFirmware\x12\x1a.pilot_api.FirmwareRequest\x1a\x19.pilot_api.FirmwareStatus0\x01\x62\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,])

_DEBUGMODE = _descriptor.EnumDescriptor(
  name='DebugMode',
  full_name='pilot_api.DebugMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DebugOff', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DebugModules', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DebugAll', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=756,
  serialized_end=813,
)
_sym_db.RegisterEnumDescriptor(_DEBUGMODE)

DebugMode = enum_type_wrapper.EnumTypeWrapper(_DEBUGMODE)
_METRICTYPE = _descriptor.EnumDescriptor(
  name='MetricType',
  full_name='pilot_api.MetricType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='IO', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='Sensor', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=815,
  serialized_end=847,
)
_sym_db.RegisterEnumDescriptor(_METRICTYPE)

MetricType = enum_type_wrapper.EnumTypeWrapper(_METRICTYPE)
_FIRMWARETARGET = _descriptor.EnumDescriptor(
  name='FirmwareTarget',
  full_name='pilot_api.FirmwareTarget',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PilotMainboard', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SmartPower', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=849,
  serialized_end=901,
)
_sym_db.RegisterEnumDescriptor(_FIRMWARETARGET)

FirmwareTarget = enum_type_wrapper.EnumTypeWrapper(_FIRMWARETARGET)
DebugOff = 0
DebugModules = 1
DebugAll = 2
IO = 0
Sensor = 1
PilotMainboard = 0
SmartPower = 1



_DEBUGMODEMSG = _descriptor.Descriptor(
  name='DebugModeMsg',
  full_name='pilot_api.DebugModeMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='mode', full_name='pilot_api.DebugModeMsg.mode', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=91,
  serialized_end=141,
)


_METRICINFO = _descriptor.Descriptor(
  name='MetricInfo',
  full_name='pilot_api.MetricInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='pilot_api.MetricInfo.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='pilot_api.MetricInfo.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='friendly_name', full_name='pilot_api.MetricInfo.friendly_name', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location', full_name='pilot_api.MetricInfo.location', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metric_type', full_name='pilot_api.MetricInfo.metric_type', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='description', full_name='pilot_api.MetricInfo.description', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='precision', full_name='pilot_api.MetricInfo.precision', index=6,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='unit', full_name='pilot_api.MetricInfo.unit', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='group', full_name='pilot_api.MetricInfo.group', index=8,
      number=9, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=144,
  serialized_end=337,
)


_FWCONFIGRESPONSE = _descriptor.Descriptor(
  name='FwConfigResponse',
  full_name='pilot_api.FwConfigResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='config', full_name='pilot_api.FwConfigResponse.config', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=339,
  serialized_end=373,
)


_IDRESPONSE = _descriptor.Descriptor(
  name='IdResponse',
  full_name='pilot_api.IdResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pilot_api.IdResponse.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uuid', full_name='pilot_api.IdResponse.uuid', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=375,
  serialized_end=415,
)


_USERREQUEST = _descriptor.Descriptor(
  name='UserRequest',
  full_name='pilot_api.UserRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='pilot_api.UserRequest.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=417,
  serialized_end=444,
)


_METRICMSG = _descriptor.Descriptor(
  name='MetricMsg',
  full_name='pilot_api.MetricMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='bool', full_name='pilot_api.MetricMsg.bool', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='double', full_name='pilot_api.MetricMsg.double', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='float', full_name='pilot_api.MetricMsg.float', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int32', full_name='pilot_api.MetricMsg.int32', index=3,
      number=4, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='int64', full_name='pilot_api.MetricMsg.int64', index=4,
      number=5, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uint32', full_name='pilot_api.MetricMsg.uint32', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uint64', full_name='pilot_api.MetricMsg.uint64', index=6,
      number=7, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='str', full_name='pilot_api.MetricMsg.str', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='value', full_name='pilot_api.MetricMsg.value',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=447,
  serialized_end=603,
)


_FIRMWAREREQUEST = _descriptor.Descriptor(
  name='FirmwareRequest',
  full_name='pilot_api.FirmwareRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='target', full_name='pilot_api.FirmwareRequest.target', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='binary', full_name='pilot_api.FirmwareRequest.binary', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='variables', full_name='pilot_api.FirmwareRequest.variables', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fwconfig', full_name='pilot_api.FirmwareRequest.fwconfig', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=605,
  serialized_end=718,
)


_FIRMWARESTATUS = _descriptor.Descriptor(
  name='FirmwareStatus',
  full_name='pilot_api.FirmwareStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='progress', full_name='pilot_api.FirmwareStatus.progress', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=720,
  serialized_end=754,
)

_DEBUGMODEMSG.fields_by_name['mode'].enum_type = _DEBUGMODE
_METRICINFO.fields_by_name['metric_type'].enum_type = _METRICTYPE
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['bool'])
_METRICMSG.fields_by_name['bool'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['double'])
_METRICMSG.fields_by_name['double'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['float'])
_METRICMSG.fields_by_name['float'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['int32'])
_METRICMSG.fields_by_name['int32'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['int64'])
_METRICMSG.fields_by_name['int64'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['uint32'])
_METRICMSG.fields_by_name['uint32'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['uint64'])
_METRICMSG.fields_by_name['uint64'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_METRICMSG.oneofs_by_name['value'].fields.append(
  _METRICMSG.fields_by_name['str'])
_METRICMSG.fields_by_name['str'].containing_oneof = _METRICMSG.oneofs_by_name['value']
_FIRMWAREREQUEST.fields_by_name['target'].enum_type = _FIRMWARETARGET
DESCRIPTOR.message_types_by_name['DebugModeMsg'] = _DEBUGMODEMSG
DESCRIPTOR.message_types_by_name['MetricInfo'] = _METRICINFO
DESCRIPTOR.message_types_by_name['FwConfigResponse'] = _FWCONFIGRESPONSE
DESCRIPTOR.message_types_by_name['IdResponse'] = _IDRESPONSE
DESCRIPTOR.message_types_by_name['UserRequest'] = _USERREQUEST
DESCRIPTOR.message_types_by_name['MetricMsg'] = _METRICMSG
DESCRIPTOR.message_types_by_name['FirmwareRequest'] = _FIRMWAREREQUEST
DESCRIPTOR.message_types_by_name['FirmwareStatus'] = _FIRMWARESTATUS
DESCRIPTOR.enum_types_by_name['DebugMode'] = _DEBUGMODE
DESCRIPTOR.enum_types_by_name['MetricType'] = _METRICTYPE
DESCRIPTOR.enum_types_by_name['FirmwareTarget'] = _FIRMWARETARGET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DebugModeMsg = _reflection.GeneratedProtocolMessageType('DebugModeMsg', (_message.Message,), {
  'DESCRIPTOR' : _DEBUGMODEMSG,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.DebugModeMsg)
  })
_sym_db.RegisterMessage(DebugModeMsg)

MetricInfo = _reflection.GeneratedProtocolMessageType('MetricInfo', (_message.Message,), {
  'DESCRIPTOR' : _METRICINFO,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.MetricInfo)
  })
_sym_db.RegisterMessage(MetricInfo)

FwConfigResponse = _reflection.GeneratedProtocolMessageType('FwConfigResponse', (_message.Message,), {
  'DESCRIPTOR' : _FWCONFIGRESPONSE,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.FwConfigResponse)
  })
_sym_db.RegisterMessage(FwConfigResponse)

IdResponse = _reflection.GeneratedProtocolMessageType('IdResponse', (_message.Message,), {
  'DESCRIPTOR' : _IDRESPONSE,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.IdResponse)
  })
_sym_db.RegisterMessage(IdResponse)

UserRequest = _reflection.GeneratedProtocolMessageType('UserRequest', (_message.Message,), {
  'DESCRIPTOR' : _USERREQUEST,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.UserRequest)
  })
_sym_db.RegisterMessage(UserRequest)

MetricMsg = _reflection.GeneratedProtocolMessageType('MetricMsg', (_message.Message,), {
  'DESCRIPTOR' : _METRICMSG,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.MetricMsg)
  })
_sym_db.RegisterMessage(MetricMsg)

FirmwareRequest = _reflection.GeneratedProtocolMessageType('FirmwareRequest', (_message.Message,), {
  'DESCRIPTOR' : _FIRMWAREREQUEST,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.FirmwareRequest)
  })
_sym_db.RegisterMessage(FirmwareRequest)

FirmwareStatus = _reflection.GeneratedProtocolMessageType('FirmwareStatus', (_message.Message,), {
  'DESCRIPTOR' : _FIRMWARESTATUS,
  '__module__' : 'pilot_api_pb2'
  # @@protoc_insertion_point(class_scope:pilot_api.FirmwareStatus)
  })
_sym_db.RegisterMessage(FirmwareStatus)



_PILOTAPI = _descriptor.ServiceDescriptor(
  name='PilotApi',
  full_name='pilot_api.PilotApi',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=904,
  serialized_end=1525,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetFwConfig',
    full_name='pilot_api.PilotApi.GetFwConfig',
    index=0,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_FWCONFIGRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetID',
    full_name='pilot_api.PilotApi.GetID',
    index=1,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_IDRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetName',
    full_name='pilot_api.PilotApi.SetName',
    index=2,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=google_dot_protobuf_dot_wrappers__pb2._BOOLVALUE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetDebugMode',
    full_name='pilot_api.PilotApi.SetDebugMode',
    index=3,
    containing_service=None,
    input_type=_DEBUGMODEMSG,
    output_type=google_dot_protobuf_dot_wrappers__pb2._BOOLVALUE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetDebugMode',
    full_name='pilot_api.PilotApi.GetDebugMode',
    index=4,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_DEBUGMODEMSG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMetricInfo',
    full_name='pilot_api.PilotApi.GetMetricInfo',
    index=5,
    containing_service=None,
    input_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    output_type=_METRICINFO,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetFloatMetric',
    full_name='pilot_api.PilotApi.GetFloatMetric',
    index=6,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=google_dot_protobuf_dot_wrappers__pb2._FLOATVALUE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetMetric',
    full_name='pilot_api.PilotApi.GetMetric',
    index=7,
    containing_service=None,
    input_type=google_dot_protobuf_dot_wrappers__pb2._STRINGVALUE,
    output_type=_METRICMSG,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='SetFirmware',
    full_name='pilot_api.PilotApi.SetFirmware',
    index=8,
    containing_service=None,
    input_type=_FIRMWAREREQUEST,
    output_type=_FIRMWARESTATUS,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_PILOTAPI)

DESCRIPTOR.services_by_name['PilotApi'] = _PILOTAPI

# @@protoc_insertion_point(module_scope)
